__author__ = 'fcanas'

import unittest
from IzVerifier.izspecs.containers.izconditions import IzConditions
from IzVerifier.izspecs.containers.izstrings import IzStrings
from IzVerifier.izspecs.containers.izvariables import IzVariables
from IzVerifier.izspecs.verifiers.seeker import Seeker
from IzVerifier.izverifier import IzVerifier
from IzVerifier.izspecs.izproperties import *


path1 = '' # path to install.xml
path2 = '' # path to install.xml
source_paths = [] # paths to code bases
class TestProduct(unittest.TestCase):
    """
    Basic testing of seeker class.
    """

    args = {
            'specs_path': path1,
            'sources': source_paths,
            'resources_path': path2,
            'specs': ['conditions', 'strings', 'variables']
        }

    def setUp(self):
        pass

    def verifyInstaller(self, args):
        """
        Runs the verifications on some installer product.
        """

        self.verifier = IzVerifier(args)
        self.seeker = Seeker(self.verifier.paths)
        langpacks = self.verifier.paths.get_langpacks()
        langpack = langpacks[0]
        self.conditions = IzConditions(self.verifier.paths.get_path('conditions'))
        self.variables = IzVariables(self.verifier.paths.get_path('variables'))
        self.strings = IzStrings(langpack[1])

    def test_verifyInstaller(self):
        pass
        #self.verifyInstaller(self.args)







__author__ = 'fcanas'

import unittest
from IzVerifier.izverifier import IzVerifier
from IzVerifier.izspecs.verifiers.dependencies import test_verify_all_dependencies
from IzVerifier.izspecs.izproperties import *


path1 = 'data/sample_installer_iz5/izpack/'
path2 = 'data/sample_installer_iz5/resources/'
source_path2 = 'data/sample_code_base'


class TestDependencies(unittest.TestCase):
    """
    Basic testing of verifier class.
    """

    def setUp(self):
        args = {
            'specs_path': path1,
            'sources': [source_path2],
            'resources_path': path2,
            'specs': ['conditions', 'strings', 'variables']
        }
        self.izv = IzVerifier(args)

    def test_verifyAllDependencies(self):
        """
        Run the full dependency verification test.
        """
        self.izv.dependency_verification()





import os
from os.path import dirname
from bs4 import BeautifulSoup
import re

__author__ = 'fcanas'


class IzPaths():
    """
    Class responsible for providing paths to specific IzPack resources and spec files.
    """
    specs = {
        'variables': 'variables.xml',
        'conditions': 'conditions.xml',
        'dynamicvariables': 'dynamic_variables.xml',
        'resources': 'resources.xml',
        'panels': 'panels.xml',
        'packs': 'packs.xml'
    }

    resources = {}
    
    def __init__(self, specs, resources):
        """
        Initialize the installer's root path.
        """

        self.set_paths(specs, resources)
        self.parse_paths()
        self.find_resources()

    def set_paths(self, specs, resources):
        """
        Takes base paths to specs and resources.
        """
        self.install = specs + '/install.xml'
        self.specs_path = path_format(specs)
        self.root = dirname(dirname(self.specs_path)) + '/'
        self.res_path = path_format(resources)

    def parse_paths(self):
        """
        Extracts paths to available izpack resources and spec files from the
        installer's install.xml spec.
        """
        self.paths = {}
        self.soup = BeautifulSoup(open(self.install))
        for spec in self.specs.keys():
            spec_file = self.find_path(spec)
            if spec_file:
                # If spec file exists
                self.paths[spec] = self.specs_path + spec_file
            else:
                # If specs are held inside install.xml
                self.paths[spec] = self.install


    def find_path(self, spec):
        """
        Find the path for the spec in the install.xml file.
        """
        path = None
        element = self.soup.find(spec)
        if element:
            child = element.find('xi:include')
            if child: # if xi:include exists, specs are external.
                path = self.strip_variables(child['href'])
            else:
                # Internal specs.
                path = None
        else:
            # No spec defined in file, assume default location.
            path = self.specs[spec]
        return path

    def get_path(self, spec):
        """
        Returns a path to the spec file, or None if there isn't any.
        """
        if not self.paths[spec]:
            return None
        else:
            return self.paths[spec]

    def find_resources(self):
        """
        Parse the install.xml resources and extract paths to available resource files.
        """
        self.resources = {}
        path = self.get_path('resources')

        if not path:
            rsoup = self.soup
        else:
            rsoup = BeautifulSoup(open(path))

        self.find_langpacks(rsoup)

    def find_langpacks(self, soup):
        langpacks = []

        for res in soup.find_all('res'):
            res_path = self.res_path + self.strip_variables(res['src'])
            self.paths[res['id']] = res_path
            if 'CustomLangPack.xml'.lower() in res['id'].lower():
                langpacks.append((res['id'], res_path))
        self.resources['langpacks'] = langpacks
        self.paths['strings'] = langpacks[0][1]

    def get_langpacks(self):
        """
        Returns a list of found langpacks in the form:
        [
            (langpack_id, langpack_path),
            ...
        ]
        """
        return self.resources['langpacks']

    def strip_variables(self, path):
        """
        Strips away variables from paths.
        TODO: proper properties substitution.
        """
        p = re.sub('\$\{.*\}','', path)
        return p

def path_format(path):
    """
    Currently ensures a path to a folder ends in a '/', and a path
    to a file does not.
    """

    if os.path.isdir(path):
        if path[-1] != '/':
            return path + '/'
    return path






__author__ = 'fcanas'

import unittest
from IzVerifier.izspecs.containers.izconditions import IzConditions
from IzVerifier.izspecs.containers.izstrings import IzStrings
from IzVerifier.izspecs.containers.izvariables import IzVariables
from IzVerifier.izverifier import IzVerifier
from IzVerifier.izspecs.izproperties import *


path1 = 'data/sample_installer_iz5/izpack/'
path2 = 'data/sample_installer_iz5/resources/'
source_path2 = 'data/sample_code_base'


class TestVerifier(unittest.TestCase):
    """
    Basic testing of verifier class.
    """

    def setUp(self):
        args = {
            'specs_path': path1,
            'sources': [source_path2],
            'resources_path': path2,
            'specs': ['conditions', 'strings', 'variables']
        }
        self.izv = IzVerifier(args)

    def test_IzPaths(self):
        """
        Testing install.xml path parsing.
        """
        specs = [('variables', 'variables.xml'),
                 ('conditions', 'conditions.xml'),
                 ('dynamicvariables', 'dynamic_variables.xml'),
                 ('resources', 'resources.xml'),
                 ('panels', 'panels.xml'),
                 ('packs', 'packs.xml')]

        self.assertTrue(self.izv != None)
        for spec in specs:
            path = self.izv.paths.get_path(spec[0])
            self.assertTrue(spec[1] in path,
                            msg=path + "!=" + spec[1])

    def test_IzConditions(self):
        """
        Testing the strings container.
        """
        conditions = self.izv.paths.get_path('conditions')
        self.assertEquals(conditions,'data/sample_installer_iz5/izpack/conditions.xml')

        izc = IzConditions(conditions)
        self.assertTrue(izc != None)

        # Test for number of keys in conditions.xml plus white list
        num = len(izc.get_keys()) - len(izc.properties[WHITE_LIST])
        self.assertEquals(num, 4, str(num) + "!=4")

    def test_langpack_paths(self):
        """
        Test that we parsed the langpack paths from resources.xml
        """
        langpacks = [('CustomLangPack.xml', 'data/sample_installer_iz5/resources/langpacks/CustomLangPack.xml'),
                     ('CustomLangPack.xml_eng', 'data/sample_installer_iz5/resources/langpacks/CustomLangPack.xml')]

        for tpack, fpack in zip(langpacks, self.izv.paths.get_langpacks()):
            self.assertTrue(tpack[1] == fpack[1], msg=tpack[1] + '!=' + fpack[1])

    def test_IzStrings(self):
        """
        Testing the strings container.
        """
        langpacks = self.izv.paths.get_langpacks()

        langpack = langpacks[0]
        izs = IzStrings(langpack[1])
        self.assertTrue(izs != None)

        # Test for number of strings
        num = len(izs.get_keys())
        self.assertEquals(num, 4, str(num) + '!=4')

    def test_IzVariables(self):
        """
        Testing the variables container.
        """
        variables = self.izv.paths.get_path('variables')
        self.assertEquals(variables,'data/sample_installer_iz5/izpack/variables.xml')

        izv = IzVariables(variables)
        self.assertTrue(izv != None)
        num = len(izv.get_keys()) - len(izv.properties[WHITE_LIST])
        self.assertEquals(num, 3, str(num) + '!=3')

    def test_verifyConditions(self):
        """
        Verify conditions in sample installer.
        """
        hits = self.izv.verify('conditions', verbosity=1)
        num = len(hits)
        self.assertTrue(num == 2)

    def test_verifyVariables(self):
        """
        Verify conditions in sample installer.
        """
        hits = self.izv.verify('variables', verbosity=1)
        num = len(hits)
        self.assertTrue(num == 4)

    def test_verifyVariables(self):
        """
        Verify conditions in sample installer.
        """
        hits = self.izv.verify('strings', verbosity=1)
        num = len(hits)
        self.assertTrue(num == 7)

    def test_verifyAll(self):
        """
        Verify all specs on sample installer.
        """
        hits = self.izv.verify_all(verbosity=1)
        num = len(hits)
        assert(num != 0)


if __name__ == '__main__':
    unittest.main()




import importlib
from IzVerifier.izspecs.verifiers.dependencies import test_verify_all_dependencies
from IzVerifier.izspecs.verifiers.seeker import Seeker
from IzVerifier.izspecs.izproperties import *
from IzVerifier.exceptions.IzVerifierException import IzArgumentsException
from IzVerifier.izspecs.izpaths import IzPaths

__author__ = 'fcanas'


class IzVerifier():
    """
    Responsible for parsing an IzPack installer's spec files and any user-specified source code,
    and finding inconsistencies with izpack's conditions, variables, strings, process panel jobs,
    etc.
    """

    def __init__(self, args):
        """
        Main entry point for IzVerifier. Args is a dictionary in this form:

        args = {
            'specs_path': path                  # Path to specs folder for installer.
            'resources_path': path              # Path to root resources folder for installer.
            'sources': [path1, path2, ...]      # Path(s) to associated source code roots.
        }
        """
        validate_arguments(args)
        self.specifications = ['conditions', 'variables', 'strings']
        self.containers = {}
        self.installer = args.get('installer')
        self.sources = args.get('sources', [])
        self.paths = IzPaths(args['specs_path'], args['resources_path'])
        self.seeker = Seeker(self.paths)

    def verify_all(self, verbosity=0):
        """
        Runs a verification for all specs.
        """
        missing = set([])

        for specification in self.specifications:
            missing |= self.verify(specification, verbosity)
        return missing


    def verify(self, specification, verbosity=0):
        """
        Runs a verification on the given izpack spec: conditions, strings, variables, etc.
        """
        container = self.get_container(specification)
        defined = container.get_keys()
        crefs = self.find_code_references(specification)
        srefs = self.find_spec_references(specification)

        cmissing = undefined(defined, crefs)

        if verbosity > 0:
            report_set(cmissing)

        smissing = undefined(defined, srefs)

        if verbosity > 0:
            report_set(smissing)

        return cmissing | smissing

    def get_container(self, specification):
        """
        Returns an izpack item container filled with specs gathered from the installer
        specified in the constructor.
        """
        if not self.containers.has_key(specification):
            return self.init_container(specification)
        else:
            return self.containers[specification]

    def init_container(self, specification):
        """
        Initialize a container to be used for verification.
        """
        module = importlib.import_module("IzVerifier.izspecs.containers.iz" + specification)
        class_ = getattr(module, 'Iz' + specification.title())
        instance = class_(self.paths.get_path(specification))
        self.containers[specification] = instance
        return instance

    def find_code_references(self, specification):
        """
        Find all source code references for specs in each container.
        :param specifications: conditions, variables, strings, or other izpack spec
        """
        container = self.get_container(specification)
        hits = self.seeker.find_references_in_source(patterns=container.properties[PATTERNS],
                                                     path_list=self.sources,
                                                     white_list_patterns=container.properties[WHITE_LIST_PATTERNS])

        return hits

    def find_spec_references(self, specification):
        """
        Find all specification xml references for specs in each container.
        :param installer:
        :param specifications:
        """
        container = self.get_container(specification)

        args = {
             'path': self.paths.root,
             'specs': container.properties[REFERENCE_SPEC_FILES],
             'filter_fn': container.has_reference,
             'attributes': container.properties[ATTRIBUTES],
             'transform_fn': container.ref_transformer
        }
        hits = self.seeker.search_specs_for_attributes(args)
        return hits

    def dependency_verification(self):
        """
        Run a conditions dependency graph search.
        """
        test_verify_all_dependencies(self)

def validate_arguments(args):
    """
    Throws exceptions if required args are missing or invalid.
    """
    if not args.has_key('specs_path'):
        raise IzArgumentsException("No Path to Installer Specs Specified")
        exit(1)
    if not args.has_key('resources_path'):
        raise IzArgumentsException("No Path to Installer Resources Specified")
        exit(1)


def undefined(key_set, tup_set):
    """
    Returns the subset of keys from tup_set not present in key_set.
    key_set is a simple set of string keys.
    up_set is a set of tuples: tup[0] is the key of that tuple.
    """
    return set([tup for tup in tup_set if not quote_remover(tup[0]) in key_set])


def unused(key_set, tup_set):
    """
    Returns the subset of key_set not present in tup_set.
    key_set is a simple set of string keys.
    up_set is a set of tuples: tup[0] is the key of that tuple.
    """
    if not tup_set:
        return key_set
    return set([key for key in key_set if not key in zip(*tup_set)[0]])


def quote_remover(key):
    """
    Extracts the actual key of the item passed in.
    TODO: seriously do this minus recursion, idiot.
    """
    if key.startswith('"') and key.endswith('"'):
        return key[1:-1]
    if '=' in key:
        key = quote_remover(''.join(key.split('=')[1:]))
    return key

def report_set(entities):
    """ Human-readable report for a set of unverified entities. """
    template = ' [ {0:.<40}{1:.>40} ] '
    template1 = ' [ {0:.<80} ] '

    for item in sorted(entities):
        if type(item) is tuple:
            print template.format(item[0], item[1])
        else:
            print template1.format(item)

